import axios from 'axios';
import { useAuth } from '../auth'; // Импортируем функцию авторизации

// Создаем экземпляр Axios с базовым URL нашего API
const apiClient = axios.create({
  baseURL: import.meta.env.VITE_API_BASE_URL || '/api',
  timeout: 10000,
  headers: {
    'Content-Type': 'application/json',
  },
});

// Храним токены отмены запросов для возможности отмены при блокировке
const cancelTokenSources = new Map();

// Интерсептор запросов - добавляет токен авторизации к каждому запросу
apiClient.interceptors.request.use(
  (config) => {
    const token = localStorage.getItem('userToken');
    if (token) {
      console.log('[API] Добавление токена к запросу:', config.url);
      config.headers['Authorization'] = `Bearer ${token}`;
    } else {
      console.log('[API] Токен не найден для запроса:', config.url);
    }
    
    // Добавляем CancelToken к запросу для возможности отмены
    const source = axios.CancelToken.source();
    config.cancelToken = source.token;
    
    // Сохраняем источник токена отмены в Map по URL запроса как ключу
    const requestKey = `${config.method}-${config.url}`;
    cancelTokenSources.set(requestKey, source);
    
    return config;
  },
  (error) => {
    console.error('[API] Ошибка в перехватчике запросов:', error);
    return Promise.reject(error);
  }
);

// Интерсептор ответов для обработки глобальных ошибок
apiClient.interceptors.response.use(
  (response) => {
    // Удаляем источник токена отмены после успешного ответа
    const requestKey = `${response.config.method}-${response.config.url}`;
    cancelTokenSources.delete(requestKey);
    return response;
  },
  (error) => {
    // Удаляем источник токена отмены при ошибке
    if (error.config) {
      const requestKey = `${error.config.method}-${error.config.url}`;
      cancelTokenSources.delete(requestKey);
    }
    
    // Проверяем на блокировку аккаунта
    if (error.response && error.response.status === 403) {
      const errorData = error.response.data;
      // Проверка на признаки блокировки аккаунта в ответе
      if (errorData && (
          errorData.blocked === true || 
          errorData.message?.includes('заблокирован') || 
          errorData.message?.includes('blocked')
      )) {
        console.error('[API] Получено уведомление о блокировке аккаунта:', errorData);
        
        // Вызываем обработчик блокировки аккаунта
        setTimeout(() => {
          const { handleAccountBlocked } = useAuth();
          if (typeof handleAccountBlocked === 'function') {
            handleAccountBlocked(errorData);
          }
        }, 0);
      }
    }
    
    if (error.response && error.response.status === 401) {
      console.error('[API] Неавторизованный запрос (401):', error.config.url);
      
      // Обходим замыкание, чтобы избежать проблем с импортом
      setTimeout(() => {
        const { logout } = useAuth();
        logout();
      }, 0);
    }
    return Promise.reject(error);
  }
);

// Функция для отмены всех активных запросов
const cancelActiveRequests = () => {
  console.log('[API] Отмена всех активных запросов. Количество: ' + cancelTokenSources.size);
  cancelTokenSources.forEach((source, key) => {
    source.cancel('Запрос отменен из-за блокировки аккаунта');
    console.log(`[API] Запрос ${key} отменен`);
  });
  cancelTokenSources.clear();
};

// Функция для явной установки токена аутентификации
const setAuthToken = (token) => {
  if (token) {
    apiClient.defaults.headers.common['Authorization'] = `Bearer ${token}`;
  } else {
    delete apiClient.defaults.headers.common['Authorization'];
  }
};

// Экспортируем функции для конкретных эндпоинтов
export default {
  // Утилитарные методы
  cancelActiveRequests,
  setAuthToken,
  
  register(userData) {
    return apiClient.post('/auth/register', userData);
  },
  login(credentials) {
    return apiClient.post('/auth/login', credentials);
  },
  getMe() {
    return apiClient.get('/auth/me');
  },
  updateUserProfile(profileData) { 
    console.log('[API] Отправка запроса на обновление профиля:', profileData);
    return apiClient.put('/users/profile', profileData);
  },
  getSuggestions() {
    return apiClient.get('/users/suggestions');
  },
  likeUser(userId) {
    return apiClient.post(`/actions/like/${userId}`);
  },
  passUser(userId) {
    return apiClient.post(`/actions/pass/${userId}`);
  },
  uploadUserProfilePhoto(formData) {
    return apiClient.post('/users/profile/photo', formData, {
      headers: {
        'Content-Type': 'multipart/form-data',
      },
    });
  },
  setMainPhoto(photoId) {
    return apiClient.put(`/users/profile/photo/${photoId}/set-main`);
  },
  deletePhoto(photoId) {
    return apiClient.delete(`/users/profile/photo/${photoId}`);
  },
  getUserConversations() {
    return apiClient.get('/conversations');
  },
  getMessagesForConversation(conversationId) {
    return apiClient.get(`/conversations/${conversationId}/messages`);
  },
  deleteMessage(conversationId, messageId) {
    return apiClient.delete(`/conversations/${conversationId}/messages/${messageId}`);
  },
  editMessage(conversationId, messageId, newText) {
    return apiClient.put(`/conversations/${conversationId}/messages/${messageId}`, { text: newText });
  },
  
  // Новый метод для получения профиля пользователя по ID
  getUserById(userId) {
    return apiClient.get(`/users/${userId}`);
  },
  
  // Новый метод для получения статистики пользователя
  getUserStats() {
    return apiClient.get('/users/stats');
  },
  
  // Новый метод для записи просмотра профиля
  recordProfileView(userId, source = 'other') {
    return apiClient.post(`/users/${userId}/view`, { source });
  },
  
  // Методы для работы с жалобами
  createReport(reportData) {
    return apiClient.post('/reports', reportData);
  },
  
  // Админские методы для жалоб
  getReports(params = {}) {
    return apiClient.get('/admin/reports', { params });
  },
  
  getReportById(reportId) {
    return apiClient.get(`/admin/reports/${reportId}`);
  },
  
  updateReportStatus(reportId, updateData) {
    return apiClient.put(`/admin/reports/${reportId}`, updateData);
  },
  
  getReportsStats() {
    return apiClient.get('/admin/reports/stats');
  }
};