import { io } from 'socket.io-client';
import { useAuth } from '@/auth';

let socket;

// Для блокировки аккаунта, добавляем состояние
let wasAccountBlocked = false;

// const SOCKET_URL = 'http://localhost:5000';
// Для разработки через Vite proxy, URL должен быть относительным к домену фронтенда
const SOCKET_URL = '/';

export const connectSocket = () => {
  const { user, isAuthenticated, handleAccountBlocked, handleAccountUnblocked } = useAuth();

  if (wasAccountBlocked) {
    console.log('[SocketService] Попытка подключения отклонена - аккаунт был заблокирован');
    return null;
  }

  if (isAuthenticated.value && user.value && user.value._id) {
    // Подключаемся только если пользователь аутентифицирован и есть ID
    socket = io(SOCKET_URL, {
      path: '/socket.io/',
      transports: ['websocket', 'polling']
    });

    console.log('[SocketService] Попытка подключения к Socket.IO серверу через Vite proxy...');

    socket.on('connect', () => {
      console.log('[SocketService] Успешно подключен к Socket.IO. Socket ID:', socket.id);
      // Отправляем ID пользователя на сервер, чтобы он знал, кто подключился
      socket.emit('addUser', user.value._id); 
    });

    socket.on('disconnect', (reason) => {
      console.log('[SocketService] Отключен от Socket.IO. Причина:', reason);
      
      // Если отключение произошло из-за блокировки аккаунта, 
      // не пытаемся автоматически переподключиться
      if (wasAccountBlocked && (reason === 'io server disconnect' || reason === 'transport close')) {
        console.log('[SocketService] Предотвращение переподключения из-за блокировки аккаунта');
        socket.disconnect();
        socket = null;
      }
    });

    socket.on('connect_error', (error) => {
      console.error('[SocketService] Ошибка подключения к Socket.IO:', error.message, error.data);
    });
    
    // ПЕРЕРАБОТАННЫЙ обработчик блокировки аккаунта
    socket.on('account_blocked', (data) => {
      console.log('[SocketService] Получено уведомление о блокировке аккаунта:', data);
      
      // Устанавливаем флаг блокировки аккаунта, но НЕ отключаем сокет
      wasAccountBlocked = true;
      
      // Вызываем обработчик блокировки для выхода из системы
      if (typeof handleAccountBlocked === 'function') {
        try {
          handleAccountBlocked(data);
        } catch (error) {
          console.error('[SocketService] Ошибка при вызове handleAccountBlocked:', error);
          window.location.href = '/login?blocked=true';
        }
      } else {
        console.error('[SocketService] handleAccountBlocked не определен!');
        window.location.reload();
      }
    });
    
    // Обработчик разблокировки аккаунта
    socket.on('account_unblocked', (data) => {
      console.log('[SocketService] Получено уведомление о разблокировке аккаунта:', data);
      
      // Сбрасываем флаг блокировки
      wasAccountBlocked = false;
      
      // Вызываем обработчик разблокировки
      if (typeof handleAccountUnblocked === 'function') {
        try {
          console.log('[SocketService] Вызов обработчика разблокировки аккаунта');
          handleAccountUnblocked(data);
        } catch (error) {
          console.error('[SocketService] Ошибка при обработке разблокировки аккаунта:', error);
          
          // В случае ошибки в обработчике, перенаправляем на страницу логина
          window.location.href = '/login?unblocked=true';
        }
      } else {
        console.log('[SocketService] Аккаунт разблокирован, но обработчик не определен');
        
        // Показываем уведомление с помощью стандартного API
        if ('Notification' in window && Notification.permission === 'granted') {
          new Notification('Аккаунт разблокирован', {
            body: data.message || 'Ваш аккаунт был разблокирован администратором.'
          });
        }
        
        // Перенаправляем на страницу логина для восстановления сессии
        window.location.href = '/login?unblocked=true';
      }
    });
    
    // Новый обработчик для глобальных уведомлений о разблокировке
    socket.on('global_user_unblocked', (data) => {
      console.log('[SocketService] Получено глобальное уведомление о разблокировке пользователя:', data);
      
      // Проверяем, относится ли это к нашему пользователю
      if (user.value && user.value._id === data.userId) {
        console.log('[SocketService] Это уведомление относится к текущему пользователю, сбрасываем флаг блокировки');
        wasAccountBlocked = false;
        
        // Вызываем обработчик разблокировки
        if (typeof handleAccountUnblocked === 'function') {
          try {
            // Создаем объект с данными разблокировки
            const unblockData = {
              message: 'Ваш аккаунт был разблокирован администратором.',
              timestamp: data.timestamp,
              userId: data.userId,
              unblockType: 'global_notification',
              unblockData: {
                canRestoreSession: true
              }
            };
            
            handleAccountUnblocked(unblockData);
          } catch (error) {
            console.error('[SocketService] Ошибка при обработке глобальной разблокировки:', error);
          }
        }
      }
    });
    
    // Обработчик глобальной блокировки 
    socket.on('global_user_blocked', (data) => {
      console.log('[SocketService] Получено глобальное уведомление о блокировке пользователя:', data);
      
      // Проверяем, относится ли это к нашему пользователю
      if (user.value && user.value._id === data.userId) {
        console.log('[SocketService] Это уведомление относится к текущему пользователю');
        wasAccountBlocked = true;
        
        // Вызываем обработчик блокировки для немедленного выхода
        if (typeof handleAccountBlocked === 'function') {
          try {
            // Создаем объект с данными блокировки
            const blockData = {
              message: 'Ваш аккаунт был заблокирован администратором.',
              timestamp: data.timestamp,
              userId: data.userId,
              blocked: true,
              blockType: 'global_notification'
            };
            
            handleAccountBlocked(blockData);
          } catch (error) {
            console.error('[SocketService] Ошибка при обработке глобальной блокировки:', error);
            // Аварийный выход в случае ошибки
            window.location.href = '/login?blocked=true';
          }
        }
      }
    });
    
    return socket;
  } else {
    console.warn('[SocketService] Не удалось подключиться: пользователь не аутентифицирован или нет ID.');
    return null;
  }
};

export const disconnectSocket = () => {
  if (socket) {
    console.log('[SocketService] Отключение от Socket.IO...');
    socket.disconnect();
    socket = null;
  }
};

export const getSocket = () => {
  // Если аккаунт был заблокирован, возвращаем null
  if (wasAccountBlocked) {
    console.warn('[SocketService] getSocket: Аккаунт заблокирован, сокет недоступен');
    return null;
  }

  if (!socket) {
    console.warn('[SocketService] getSocket вызван, но сокет не инициализирован. Попытка подключения...');
  }
  return socket;
};

// Новая функция для проверки статуса блокировки
export const isAccountBlocked = () => {
  return wasAccountBlocked;
};

// Новая функция для сброса флага блокировки аккаунта
export const resetAccountBlockedFlag = () => {
  console.log('[SocketService] Сброс флага блокировки аккаунта');
  wasAccountBlocked = false;
};